searchData={"items":[{"type":"module","title":"Enuma","doc":"Enuma is a library for defining and working with Rust like Enums in Elixir.","ref":"Enuma.html"},{"type":"module","title":"Example - Enuma","doc":"```elixir\ndefmodule MyEnum do\n  use Enuma\n\n  defenum do\n    item :foo\n    item :bar, args: [integer()]\n    item :baz, args: [String.t()]\n  end\nend\n```\n\nEnuma will create macros for each item, allowing you to match on the enum values and check if a value is of a specific type.\n\n```elixir\niex> require MyEnum\nMyEnum\n\niex> MyEnum.foo() = :foo\ntrue\n\niex> MyEnum.bar(x) = {:bar, 1}\n{:bar, 1}\n```\n\nIt will also generate macros for checking if a value is of a specific type. (e.g `MyEnum.is_foo(value)`).\nThese macros can be used in pattern matching and guards.","ref":"Enuma.html#module-example"},{"type":"macro","title":"Enuma.defenum/1","doc":"Defines an enum with the given items.","ref":"Enuma.html#defenum/1"},{"type":"function","title":"Enuma.from_string/2","doc":"","ref":"Enuma.html#from_string/2"},{"type":"function","title":"Enuma.from_string!/2","doc":"","ref":"Enuma.html#from_string!/2"},{"type":"macro","title":"Enuma.is_valid/2","doc":"","ref":"Enuma.html#is_valid/2"},{"type":"macro","title":"Enuma.item/2","doc":"Defines an item with the given name and options.","ref":"Enuma.html#item/2"},{"type":"macro","title":"Options - Enuma.item/2","doc":"* `:args` - a list of arguments to be passed to the item macro","ref":"Enuma.html#item/2-options"},{"type":"function","title":"Enuma.to_string/2","doc":"","ref":"Enuma.html#to_string/2"},{"type":"function","title":"Enuma.valid?/2","doc":"","ref":"Enuma.html#valid?/2"},{"type":"module","title":"Enuma.Ecto","doc":"Ecto type for Enuma Enums.","ref":"Enuma.Ecto.html"},{"type":"module","title":"Using Enuma.Ecto with Ecto schemas - Enuma.Ecto","doc":"The `Enuma.Ecto` type allows you to use Enuma enums in your Ecto schemas.\nIt supports two storage formats: `:string` and `:map`.","ref":"Enuma.Ecto.html#module-using-enuma-ecto-with-ecto-schemas"},{"type":"module","title":"Basic Usage - Enuma.Ecto","doc":"To use an Enuma enum in your Ecto schema:\n\n```elixir\ndefmodule StatusEnum do\n  use Enuma\n\n  defenum do\n    item :active\n    item :inactive\n  end\nend\n\ndefmodule MySchema do\n  use Ecto.Schema\n\n  schema \"my_table\" do\n    field :status, Enuma.Ecto, type: StatusEnum, ecto_type: :string\n  end\nend\n```","ref":"Enuma.Ecto.html#module-basic-usage"},{"type":"module","title":"Storage Formats - Enuma.Ecto","doc":"#### String Format (`:string`)\n\nWhen using `:string` as the `ecto_type`, the enum will be stored as a string in the database.\nThis format only supports simple enum values (atoms without arguments).\n\n```elixir\nfield :status, Enuma.Ecto, type: StatusEnum, ecto_type: :string\n```\n\nWith this configuration:\n- `:pending` will be stored as `\"pending\"` in the database\n- Complex enum values (tuples with arguments) are not supported and will result in an error\n\n#### Map Format (`:map`)\n\nWhen using `:map` as the `ecto_type`, the enum will be stored as a JSON map in the database.\nThis format supports both simple enum values and complex enum values with arguments.\n\n```elixir\nfield :message, Enuma.Ecto, type: MessageEnum, ecto_type: :map\n```\n\nWith this configuration items will be stored as maps with a `\"key\"` and `\"values\"` item.","ref":"Enuma.Ecto.html#module-storage-formats"},{"type":"module","title":"Ecto Changesets - Enuma.Ecto","doc":"When working with changesets, you can use the enum values directly:\n\n```elixir\ndef changeset(schema, params) do\n  schema\n  |> cast(params, [:status])\n  |> validate_required([:status])\nend\n```\n\nYou can then pass enum values in your params:\n\n```elixir\nMySchema.changeset(%MySchema{}, %{status: StatusEnum.active()})\n# Or with complex values\nMySchema.changeset(%MySchema{}, %{message: MyComplexEnum.move(1, 2)})\n```","ref":"Enuma.Ecto.html#module-ecto-changesets"},{"type":"function","title":"Enuma.Ecto.cast/2","doc":"","ref":"Enuma.Ecto.html#cast/2"},{"type":"function","title":"Enuma.Ecto.dump/3","doc":"","ref":"Enuma.Ecto.html#dump/3"},{"type":"function","title":"Enuma.Ecto.init/1","doc":"","ref":"Enuma.Ecto.html#init/1"},{"type":"function","title":"Enuma.Ecto.load/3","doc":"","ref":"Enuma.Ecto.html#load/3"},{"type":"function","title":"Enuma.Ecto.type/1","doc":"","ref":"Enuma.Ecto.html#type/1"},{"type":"extras","title":"Enuma","doc":"# Enuma\n\n[![Hex.pm](https://img.shields.io/hexpm/v/enuma.svg)](https://hex.pm/packages/enuma)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/enuma)\n\nRust-like Enums for Elixir. Enuma makes it easy to define rich enumeration types similar to Rust, while maintaining Elixir's pattern matching capabilities.","ref":"readme.html"},{"type":"extras","title":"Features - Enuma","doc":"- Define structured enum types with named variants\n- No runtime performance overhead due to compile-time evaluation of macros\n- Variants can contain no data or structured data\n- Pattern matching on enum variants\n- Guard-compatible matching with `is_*` macros\n- Optional Ecto integration for database storage","ref":"readme.html#features"},{"type":"extras","title":"Installation - Enuma","doc":"Add `enuma` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:enuma, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Enuma","doc":"Enuma uses atoms for simple items and tuples for complex items (items with arguments). All Enuma\nenuma helpers are macros so it will not have any runtime impact on performance.","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - Enuma","doc":"```elixir\ndefmodule Shape do\n  use Enuma\n\n  defenum do\n    item :circle, args: [float()]   # Circle with radius\n    item :rectangle, args: [float(), float()]  # Rectangle with width and height\n    item :triangle\n  end\nend\n\n# Creating enum values\ncircle = Shape.circle(5.0)  # {:circle, 5.0}\nrect = Shape.rectangle(4.0, 3.0)  # {:rectangle, 4.0, 3.0}\ntriangle = Shape.triangle()  # :triangle\n\n# Require is needed to use the Enuma helpers since they are macros\nrequire Shape\n\ndef calculate_area(shape) do\n  case shape do\n    Shape.circle(r) -> :math.pi() * r * r\n    Shape.rectangle(w, h) -> w * h\n    Shape.triangle() -> raise \"Area calculation for triangle not implemented\"\n  end\nend\n\n# Guard expressions\ndef print_shape(shape) when Shape.is_circle(shape) do\n  Shape.circle(radius) = shape\n  IO.puts(\"Circle with radius #{radius}\")\nend\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Ecto Integration - Enuma","doc":"Enuma provides Ecto integration for storing enum values in the database:\n\n```elixir\ndefmodule DrawingObject do\n  use Ecto.Schema\n\n  schema \"drawing_objects\" do\n    # Using map serialization to support all shape variants including those with parameters\n    # Using string serialization only works with enums only holding simple types (no arguments)\n    field :shape, Enuma.Ecto, type: Shape, ecto_type: :map\n\n    timestamps()\n  end\nend\n```\n\nThe `:map` serialization format supports all variant types, storing them in the database as JSON-compatible maps:\n\n- Simple variants like `:triangle` are stored as `%{\"key\" => \"triangle\", \"values\" => []}`\n- Complex variants like `{:circle, 5.0}` are stored as `%{\"key\" => \"circle\", \"values\" => [5.0]}`\n- Multi-parameter variants like `{:rectangle, 4.0, 3.0}` are stored as `%{\"key\" => \"rectangle\", \"values\" => [4.0, 3.0]}`\n\nUsing with changesets:\n\n```elixir\nimport Ecto.Changeset\n\n# Valid values include all variants from your enum definition\nparams = %{\n  shape: Shape.circle(5.0)\n}\n\n# Cast and validate enum values\nchangeset =\n  %DrawingObject{}\n  |> cast(params, [:shape])\n  |> validate_required([:shape])\n```\n\nRead more about Ecto integration in the `Enuma.Ecto` module documentation.","ref":"readme.html#ecto-integration"},{"type":"extras","title":"Documentation - Enuma","doc":"Complete documentation is available at [https://hexdocs.pm/enuma](https://hexdocs.pm/enuma).","ref":"readme.html#documentation"},{"type":"extras","title":"License - Enuma","doc":"MIT License. See LICENSE file for details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}